﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinRadialMenu;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.AppStyling;
using System.IO;

namespace Radial_Menu_Features
{
    public partial class Form1 : Form
    {
        #region Constructor
        public Form1()
        {
            InitializeComponent();
        }     
        #endregion // Constructor

        #region Helper Methods

        #region ApplyBoldText
        private void ApplyBoldText(bool isChecked)
        {
            this.lblSampleText.Appearance.FontData.Bold = DefaultableBooleanFromBoolean(isChecked);
        }
        #endregion // ApplyBoldText

        #region ApplyFontColor
        private void ApplyFontColor(RadialMenuColorTool radialMenuColorTool)
        {
            this.lblSampleText.Appearance.ForeColor = radialMenuColorTool.Color;
        }
        #endregion // ApplyFontColor

        #region ApplyItalicText
        private void ApplyItalicText(bool isChecked)
        {
            this.lblSampleText.Appearance.FontData.Italic = DefaultableBooleanFromBoolean(isChecked);
        }        
        #endregion // ApplyItalicText

        #region ApplyHighlightColor
        private void ApplyHighlightColor(RadialMenuColorTool radialMenuColorTool)
        {
            this.lblSampleText.Appearance.BackColor = radialMenuColorTool.Color;
        }
        #endregion // ApplyHighlightColor

        #region ApplyStrikeText
        private void ApplyStrikeText(bool isChecked)
        {
            this.lblSampleText.Appearance.FontData.Strikeout = DefaultableBooleanFromBoolean(isChecked);
        } 
        #endregion // ApplyStrikeText

        #region ApplyTextAlignment
        private void ApplyTextAlignment(HAlign hAlign)
        {
            this.lblSampleText.Appearance.TextHAlign = hAlign;
        } 
        #endregion // ApplyTextAlignment

        #region ApplyUnderlineText
        private void ApplyUnderlineText(bool isChecked)
        {
            this.lblSampleText.Appearance.FontData.Underline = DefaultableBooleanFromBoolean(isChecked);
        } 
        #endregion // ApplyUnderlineText

        #region DefaultableBooleanFromBoolean
        private static DefaultableBoolean DefaultableBooleanFromBoolean(bool boolean)
        {
            return boolean ? DefaultableBoolean.True : DefaultableBoolean.False;
        } 
        #endregion // DefaultableBooleanFromBoolean

        #region PopulateIslCombo
        private void PopulateIslCombo()
        {
            this.cboISL.Items.Add("(None)", "(None)");

            Type thisType = this.GetType();
            string[] resources = thisType.Assembly.GetManifestResourceNames();
            foreach (string resource in resources)
            {
                if (resource.EndsWith(".isl"))
                {
                    string[] resourceParts = resource.Split('.');
                    string resourceName = resourceParts[2];
                    this.cboISL.Items.Add(resource, resourceName);
                }
            }

            this.cboISL.SelectedIndex = 0;
        }  
        #endregion // PopulateIslCombo

        #region ProcessToolClick
        private void ProcessToolClick(RadialMenuTool tool)
        {
            switch (tool.Key)
            {
                #region Color
                case "Color":
                    {
                        // What happens here depends on the last toggled child tool. So process the click
                        // as though the LastToggledChildTool were clicked. 
                        RadialMenuColorTool lastToggledChildTool = tool.LastToggledChildTool as RadialMenuColorTool;
                        if (null != lastToggledChildTool)
                        {
                            this.ProcessToolClick(lastToggledChildTool);
                        }
                        break;
                    }
                #endregion // Color

                #region Copy
                case "Copy":
                    {
                        Clipboard.SetText(this.lblSampleText.Text);
                        break;
                    }
                #endregion // Copy

                #region Cut
                case "Cut":
                    {
                        Clipboard.SetText(this.lblSampleText.Text);
                        break;
                    }
                #endregion // Cut

                #region Paste
                case "Paste":
                    {
                        string pasteText = Clipboard.GetText();
                        if (false == string.IsNullOrEmpty(pasteText))
                            this.lblSampleText.Text = pasteText;

                        break;
                    }
                #endregion // Paste

                #region Font Name
                case "Font Name":
                    {
                        this.UpdateFontName((RadialMenuFontListTool)tool);
                        break;
                    }
                #endregion // Font Name

                #region Font Size
                case "Font Size":
                    {
                        this.UpdateFontSize((RadialMenuNumericTool)tool);
                        break;
                    }
                #endregion // Font Name

                #region FontColor
                case "FontColor":
                    {
                        this.ApplyFontColor((RadialMenuColorTool)tool);
                        break;
                    }
                #endregion // FontColor

                #region HighlightColor
                case "HighlightColor":
                    {
                        this.ApplyHighlightColor((RadialMenuColorTool)tool);
                        break;
                    }
                #endregion // HighlightColor

                #region FillColor
                case "FillColor":
                    {
                        // UltraLabel doesn't support a fill color, so do nothing. 
                        break;
                    }
                #endregion // FillColor
            }
        }
        #endregion // ProcessToolClick

        #region UpdateFontName
        private void UpdateFontName(RadialMenuFontListTool fontListTool)
        {
            this.lblSampleText.Appearance.FontData.Name = fontListTool.SelectedValue.ToString();
        } 
        #endregion // UpdateFontName

        #region UpdateFontSize
        private void UpdateFontSize(RadialMenuNumericTool numericTool)
        {
            this.lblSampleText.Appearance.FontData.SizeInPoints = numericTool.Value;
        }
        #endregion // UpdateFontSize        

        #region ShowRadialMenu
        private void ShowRadialMenu()
        {
            // Start at the upper left corner of the panel. 
            Point point = this.pnlRadialMenu.Location;

            // Find the center of the panel. 
            point.Offset(this.pnlRadialMenu.Width / 2, this.pnlRadialMenu.Height / 2);

            // Offset left and up by the size of the radial menu. 
            point.Offset(-this.ultraRadialMenu1.MenuSettings.Size.Width / 2, -this.ultraRadialMenu1.MenuSettings.Size.Height / 2);

            // Translate to a screen point. 
            point = this.PointToScreen(point);

            // Show the menu centered in the panel. 
            this.ultraRadialMenu1.Show(this, point);
        }
        #endregion // ShowRadialMenu 

        #endregion // Helper Methods

        #region Event Handlers

        #region ultraRadialMenu1_ToolClick
        private void ultraRadialMenu1_ToolClick(object sender, Infragistics.Win.UltraWinRadialMenu.RadialMenuToolClickEventArgs e)
        {
            string key = e.Tool.Key;

            switch (key)
            {
                case "FontColor":                    
                case "HighlightColor":               
                case "FillColor":
                    // Clicking directly on a color category automatically expands the child colors and no other action is neeeded.
                    return;               
            }

            RadialMenuColorWellTool colorWellTool = e.Tool as RadialMenuColorWellTool;
            if (null != colorWellTool)
            {
                // If the tool is a color well, then we need to change either the BackColor or ForeColor, depending on the
                // ancestor color tool. The color well itself doesn't matter, it's the category it's under that's important. 
                // So find the color tool that is it's ancestor and process the click as though that tool
                // were clicked. 
                RadialMenuColorTool colorCategoryTool = colorWellTool.FindAncestor(typeof(RadialMenuColorTool)) as RadialMenuColorTool;
                if (null != colorCategoryTool)
                {
                    this.ProcessToolClick(colorCategoryTool);
                    return;
                }
            }

            this.ProcessToolClick(e.Tool);
        }
        #endregion // ultraRadialMenu1_ToolClick        

        #region ultraRadialMenu1_ListToolSelectedValueCommitted
        private void ultraRadialMenu1_ListToolSelectedValueCommitted(object sender, RadialMenuListToolSelectedValueCommittedEventArgs e)
        {
            switch (e.Tool.Key)
            {
                #region Font Name
                case "Font Name":
                    {
                        this.UpdateFontName((RadialMenuFontListTool)e.Tool);
                        break;
                    }
                #endregion // Font Name
            }
        } 
        #endregion // ultraRadialMenu1_ListToolSelectedValueCommitted

        #region ultraRadialMenu1_ToolCheckedChanged
        private void ultraRadialMenu1_ToolCheckedChanged(object sender, RadialMenuToolCheckedChangedEventArgs e)
        {
            switch (e.Tool.Key)
            {
                #region Align Left
                case "Align Left":
                    {
                        if (e.Tool.Checked)
                            this.ApplyTextAlignment(HAlign.Left);
                    }
                    break;
                #endregion // Align Left

                #region Align Right
                case "Align Right":
                    {
                        if (e.Tool.Checked)
                            this.ApplyTextAlignment(HAlign.Right);
                    }
                    break;
                #endregion // Align Right

                #region Center
                case "Center":
                    {
                        if (e.Tool.Checked)
                            this.ApplyTextAlignment(HAlign.Center);

                        // Synchronize the Checked state of the "ParentCenter" tool.
                        // But don't fire the events, because we don't want this getting cancelled
                        // in ToolCheckedChanging. 
                        RadialMenuTool parentCenterTool = e.Tool.ParentTool;
                        parentCenterTool.SetChecked(e.Tool.Checked, false);

                    }
                    break;
                #endregion // Center

                #region ParentCenter
                case "ParentCenter":
                    {
                        // When ParentCenter is checked, synchronize the child "Center" tool. 
                        RadialMenuTool centerTool = e.Tool.FindToolByKey("Center");
                        centerTool.Checked = e.Tool.Checked;
                    }
                    break;
                #endregion // ParentCenter

                #region Bold
                case "Bold":
                    {
                        // There are two bold tools, one parent and one child. Keep them in synch.

                        // First, check to see if this tool's parent is the "Bold" tool. 
                        RadialMenuTool boldTool = e.Tool.ParentTool;
                        if (boldTool.Key == "Bold")
                            boldTool.Checked = e.Tool.Checked;

                        // Now check to see if it has a child whose key is "Bold"
                        boldTool = e.Tool.FindToolByKey("Bold");
                        if (null != boldTool)
                            boldTool.Checked = e.Tool.Checked;

                        this.ApplyBoldText(e.Tool.Checked);
                    }
                    break; 
                #endregion // Bold

                #region Italic
                case "Italic":
                    {
                        this.ApplyItalicText(e.Tool.Checked);
                    }
                    break;    
                #endregion // Italic            

                #region Strike
                case "Strike":
                    {
                        this.ApplyStrikeText(e.Tool.Checked);
                    }
                    break; 
                #endregion // Strike

                #region Underline
                case "Underline":
                    {
                        this.ApplyUnderlineText(e.Tool.Checked);
                    }
                    break; 
                #endregion // Underline

            }
        } 
        #endregion // ultraRadialMenu1_ToolCheckedChanged

        #region ultraRadialMenu1_ToolCheckedChanging
        private void ultraRadialMenu1_ToolCheckedChanging(object sender, RadialMenuToolCheckedChangingEventArgs e)
        {
            switch (e.Tool.Key)
            {
                #region ParentCenter
                case "ParentCenter":
                    {
                        // If the current state is true, it means the user is trying to change it
                        // to false and we cannot allow that. 
                        if (e.Tool.Checked == true)
                            e.Cancel = true;                       
                    }
                    break;
                #endregion // ParentCenter
            }
        }       
        #endregion // ultraRadialMenu1_ToolCheckedChanging

        #region ultraRadialMenu1_ToolValueChanged
        private void ultraRadialMenu1_ToolValueChanged(object sender, RadialMenuToolValueChangedEventArgs e)
        {            
            switch (e.Tool.Key)
            {
                #region Font Size
                case "Font Size":
                    {
                        this.UpdateFontSize((RadialMenuNumericTool)e.Tool);
                        break;
                    }
                #endregion // Font Name
            }
        }
        #endregion // ultraRadialMenu1_ToolValueChanged

        #region Form1_Load
        private void Form1_Load(object sender, EventArgs e)
        {
            this.PopulateIslCombo();

            // Show the radial menu in the center of the pnlRadialMenu panel.
            this.ShowRadialMenu();
        }        
        #endregion // Form1_Load

        #region pnlRadialMenu_Resize
        private void pnlRadialMenu_Resize(object sender, EventArgs e)
        {
            // Show the radial menu in the center of the pnlRadialMenu panel.
            this.ShowRadialMenu();
        }
        #endregion // pnlRadialMenu_Resize

        #region pnlRadialMenu_LocationChanged
        private void pnlRadialMenu_LocationChanged(object sender, EventArgs e)
        {
            // Show the radial menu in the center of the pnlRadialMenu panel.
            this.ShowRadialMenu();
        }
        #endregion // pnlRadialMenu_LocationChanged            

        #region Form1_LocationChanged
        private void Form1_LocationChanged(object sender, EventArgs e)
        {
            this.ShowRadialMenu();
        }
        #endregion // Form1_LocationChanged

        #region btnReset_Click
        private void btnReset_Click(object sender, EventArgs e)
        {
            this.lblSampleText.Appearance = null;
        } 
        #endregion // btnReset_Click                      

        #region tbRotation_ValueChanged
        private void tbRotation_ValueChanged(object sender, EventArgs e)
        {
            TrackBarWithLabel tb = (TrackBarWithLabel)sender;
            this.ultraRadialMenu1.MenuSettings.RotationInDegrees = tb.Value;
        } 
        #endregion // tbRotation_ValueChanged

        #region tbRotationPercentage_ValueChanged
        private void tbRotationPercentage_ValueChanged(object sender, EventArgs e)
        {
            TrackBarWithLabel tb = (TrackBarWithLabel)sender;
            this.ultraRadialMenu1.MenuSettings.RotationAsPercentageOfWedge = tb.Value / 100F;
        } 
        #endregion // tbRotationPercentage_ValueChanged

        #region tbWedgeCount_ValueChanged
        private void tbWedgeCount_ValueChanged(object sender, EventArgs e)
        {
            TrackBarWithLabel tb = (TrackBarWithLabel)sender;
            this.ultraRadialMenu1.MenuSettings.WedgeCount = tb.Value;
        } 
        #endregion // tbWedgeCount_ValueChanged

        #region tbCircumference_ValueChanged
        private void tbCircumference_ValueChanged(object sender, EventArgs e)
        {
            TrackBarWithLabel tb = (TrackBarWithLabel)sender;
            this.ultraRadialMenu1.MenuSettings.Size = new Size(tb.Value, tb.Value);

            // Re-center the menu. 
            this.ShowRadialMenu();
        } 
        #endregion // tbCircumference_ValueChanged

        #region cboISL_ValueChanged
        private void cboISL_ValueChanged(object sender, EventArgs e)
        {
            UltraComboEditor combo = (UltraComboEditor)sender;            
            string resourceName = combo.Value.ToString();

            Type thisType = this.GetType();
            Stream stream = thisType.Assembly.GetManifestResourceStream(resourceName);
            if (null != stream)
            {
                StyleManager.Load(stream, true, this.ultraRadialMenu1.StyleLibraryName);
            }
            else
            {
                StyleManager.Reset(this.ultraRadialMenu1.StyleLibraryName);
            }
        }
        #endregion // cboISL_ValueChanged
       

        #endregion // Event Handlers
    }
}
